#pragma once

#include "AbstractUiInterface.h"

#include <memory>
#include <map>

class DesktopManager;
class GlxInterface;
using GlxInterfacePtr = std::unique_ptr<GlxInterface>;

class XcbEventInterface;
using XcbEventInterfacePtr = std::unique_ptr<XcbEventInterface>;

class XcbWindowInterface;
using XcbWindowInterfacePtr = std::unique_ptr<XcbWindowInterface>;

struct xcb_connection_t;
struct xcb_expose_event_t;
struct _XDisplay;

namespace mt
{
    class Window;
}

class XcbInterface : public AbstractUIInterface
{
public:
    XcbInterface();

    ~XcbInterface();

    void initialize(DesktopManager* desktopManager) override;

    void loop(DesktopManager* desktopManager) override;

    void shutDown() override;

    void showWindow(mt::Window* window) override;

    void addWindow(mt::Window* window, DesktopManager* desktopManager) override;

    void CreateOpenGlDrawable(mt::Window* window);

private:
    void onPaint(DesktopManager* desktopManager);

    void InitializeOpenGl();

    void Connect();

    void UpdateScreen(DesktopManager* desktopManager);

    void OnExposeEvent(xcb_expose_event_t* event, DesktopManager* desktopManager);

    uint32_t GetEventMask();

    void MapWindow(mt::Window* window);

    void CreateGraphicsContext(DesktopManager* desktopManager);

    void onEventsDispatched(DesktopManager* desktopManager);
    void OnLoopCompleted(DesktopManager* desktopManagers);

private:
    xcb_connection_t* mConnection;
    _XDisplay* mX11Display;
    GlxInterfacePtr mGlxInterface;
    XcbEventInterfacePtr mXcbEventInterface {nullptr};
    XcbWindowInterfacePtr mXcbWindowInterface {nullptr};
};
